define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"thornberry/app/Component/_EditorComponent",
	"thornberry/app/Component/_DataFormMixin",
	"thornberry/app/Component/_TableFieldEditorMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/BalanceTransferEditor.html",
	"dojo/text!./Templates/PatientSection.html",
	"dojo/text!./Templates/ClaimSection.html",
	"thornberry/object/TableManager",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/on",
    "dojo/Deferred",
    "dojo/promise/all",
	"dojo/when",
	"dojo/date",
	"dijit/form/_TextBoxMixin",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/CellWidget",
	"dojo/store/Memory",
	"thornberry/app/utils",
	"dijit/form/ValidationTextBox",
	"dijit/form/Textarea",
	"dijit/form/CurrencyTextBox",
	"dijit/form/Select",
	"dijit/form/DateTextBox",
	"dijit/form/CheckBox"
],function(
	declare,
	lang,
	arrayUtils,
	_EditorComponent,
	_DataFormMixin,
	_TableFieldEditorMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	patientSectionTemplate,
	claimSectionTemplate,
	TableManager,
	domStyle,
	domClass,
	on,
	Deferred,
	all,
	when,
	date,
	_TextBoxMixin,
	Grid,
	Cache,
	CellWidget,
	Memory,
	utils
) {
	return declare([_EditorComponent, _DataFormMixin, _TemplatedMixin, _WidgetsInTemplateMixin, _TableFieldEditorMixin], {
		templateString: template,
		amounts: null,
		_transferAllDeferreds: {},

		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

			t.amounts = {};

			t.templateString = lang.replace(template,{
				patientSectionTemplate: patientSectionTemplate,
				claimSectionTemplate:   claimSectionTemplate
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			var amountDecorator = function() {
				return "<input data-dojo-type='dijit/form/CurrencyTextBox' data-dojo-attach-point='amountField' data-dojo-props=\"selectOnClick: true, currency:'USD', maxLength:13\">";
			};
			var amountSetCellValue = function(gridData, storeData, cellWidget) {
				var id = cellWidget.cell.row.id;
				var data = t.chargeGrid.store.get(id);
				var amountField = cellWidget.amountField;
				if (amountField._cnnt) {
					amountField._cnnt.remove();
				}

				amountField._onChangeActive = false;
				amountField.set("value", data.amount || 0);
				amountField.set("disabled", true);
				amountField._onChangeActive = true;

				t.styleAmountField(amountField);

				amountField._cnnt = on(amountField, "change", function() {
					try {
						//if (t.chargeGrid.cellWidget.getCellWidget(id, "amount")) {
						t.onAmountChanged(id, amountField.get("value"));
					} catch (e) {
						console.error(e);
					}
				});

				// TODO: selectOnClick isn't working in these cell widgets
				//amountField._cnnt = on(amountField, 'mousedown, touchstart',  lang.hitch(t, function (event) {
				//	debugger;
				//}));
			};

			var transferDecorator = function() {
				return "<input data-dojo-type='dijit/form/CheckBox' data-dojo-attach-point='transferField'>";
			};
			var transferSetCellValue = function(gridData, storeData, cellWidget) {
				var id = cellWidget.cell.row.id;
				var data = t.chargeGrid.store.get(id);
				var transferField = cellWidget.transferField;
				if (transferField._cnnt) {
					transferField._cnnt.remove();
				}

				transferField._onChangeActive = false;
				transferField.set("value", (typeof t.amounts[id] !== "undefined"));
				transferField._onChangeActive = true;

				transferField._cnnt = on(transferField, "change", function() {
					t.onTransferChanged(id, transferField.get("checked"));
				});
			};
			
			var chargeStructure = [
				{ id: "type", field: "type", name: "Type" },
				{ id: "description", field: "description", name: "Description" },
				{ id: "chargeDate", field: "chargeDate", name: "Date", formatter: utils.formatters.date("chargeDate") },
				{ id: "chargeCode", field: "chargeCode", name: "Charge Code" },  
				{ id: "net", field: "net", name: "Net", formatter: utils.formatters.dollars("net"), style: "text-align: right" },
				{ id: "balance", field: "balance", name: "Balance", formatter: utils.formatters.dollars("balance"), style: "text-align: right" },
				{ id: "hcpcsCode", field: "hcpcsCode", name: "HCPCS Code" },
				{ id: "transfer", field: "transfer", name: "Transfer?", widgetsInCell: true, decorator: transferDecorator, setCellValue: transferSetCellValue, width: "65px" },
				{ id: "amount", field: "amount", name: "Amount", widgetsInCell: true, decorator: amountDecorator, setCellValue: amountSetCellValue, width: "200px" }
			];
			t.chargeGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: chargeStructure,
				autoHeight: true,
				modules: [CellWidget]
			});
			t.chargeGrid.connect(t.chargeGrid.cellWidget, "onCellWidgetCreated", function(widget, column) {
				// Not all widgets are necessarily created at once; this catches those created
				// when rows scroll into view.
				if (widget.amountField) {
					t.styleAmountField(widget.amountField);
				}
			});
			t.chargeGrid.startup();
			t.chargeGridContainer.appendChild(t.chargeGrid.domNode);

			t.addLogic("transferAllCharges", function(name, value) {
				for (var chargeId in t._transferAllDeferreds) {
					t._transferAllDeferreds[chargeId].reject();
				}
				
				t._transferAllDeferreds = {};
				var transferAllPromises = [];

				arrayUtils.forEach(t.chargeGrid.rows(), function(row) {
					transferWidget = row.grid.cellWidget.getCellWidget(row.id, "transfer");
					if (transferWidget) {
						var transferAllDeferred = new Deferred();
						transferAllPromises.push(transferAllDeferred.promise);
						t._transferAllDeferreds[row.id] = transferAllDeferred;
						
						transferWidget.transferField.set("checked", value == "on");
					}
				});

				if (transferAllPromises.length > 0) {
					when(all(transferAllPromises), function() {
						t.updateAmountFieldStates();
						t.updateTotalAmount();
					});
				}
			}, true);
			
			when(TableManager.get("Tables.ArAccountStatus"), function(table) {
				t.accountStatus.setStore(table);
			});

			t.addRequirement(["transferDate", "insurance"], true);
			t.addDisabled(["totalAmount"], true);
			
			t.addRequirement(["dcn"], function() {
				if (t.props.value && t.props.value.account) {
					if (t.props.value.account.dcnRequired == 1) {
						return true;
					} else {
						return false;
					}
				}
			});

			t.addTableFilter("insurance", function(name, tableItem, value) {
				return tableItem.id != t.props.value.insurance.id;
			});
		},

		refreshRendering: function() {
			var t = this;

			if (t.value !== t.props.value) {
				t.value = t.props.value;
				t.set("value", t.props.value);

				var chargeStore = new Memory({data:t.props.value ? t.props.value.charges.charges : []});
				t.chargeGrid.setStore(chargeStore);

				t.setTable("insurance", (t.props.value ? t.props.value.policies : []), ["insurance"]);

				t.updateAmountFieldStates();
			}
			
			for (var chargeId in t._transferAllDeferreds) {
				t._transferAllDeferreds[chargeId].reject();
			}
			t._transferAllDeferreds = {};

			t.chargeGrid.model.clearCache();
			t.chargeGrid.body.refresh();

			t.getField("transferDate").set("constraints", {
				min: (t.value && t.value.account && t.value.account.lastCloseDate ? date.add(new Date(t.value.account.lastCloseDate), "day", 1) : null),
				max: new Date()
			});

			var disabled = t.props.disabled;
			domStyle.set(t.accountStatusNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.accountStatusFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.dcnNode,						"display", (disabled ? "" : "none"));
			domStyle.set(t.dcnFieldContainer,			"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpDateNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpDateFieldContainer,	"display", (disabled ? "none" : ""));
			domStyle.set(t.followUpUserNode,			"display", (disabled ? "" : "none"));
			domStyle.set(t.followUpUserFieldContainer,	"display", (disabled ? "none" : ""));

			domStyle.set(t.noInsuranceMessage,			"display", t.getField("insurance").options.length == 1 ? "" : "none");

			domStyle.set(t.replacedClaimContainer,		"display", (t.props.value && t.props.value.claim.replacedClaim != "" ? "" : "none"));
			domStyle.set(t.replacedByClaimContainer,	"display", (t.props.value && t.props.value.claim.replacedByClaims != "" ? "" : "none"));

			t.inherited(arguments);
		},

		updateAmountFieldStates: function() {
			var t = this;
			
			arrayUtils.forEach(t.chargeGrid.rows(), function(row) {
				var amountWidget = t.chargeGrid.cellWidget.getCellWidget(row.id, "amount");
				if (amountWidget) {
					var amountField = amountWidget.amountField;
					t.styleAmountField(amountField);
					
					// force display (formatted) value to show
					// from NumberTextBox._setBlurValue(), except with priorityChange = false
					var val = lang.hitch(lang.delegate(amountField, { focused: true }), "get")('value'); // parse with editOptions
					amountField._setValueAttr(val, false);
				}
			});
		},

		styleAmountField: function(amountField) {
			var t = this;
			var disabled = amountField.get("disabled");
			
			var val = amountField.get("value");
			var isNull = (val === "" || /^\s+$/.test(val) || /*NaN check*/ val != val || val === null || typeof val === "undefined" || (typeof val === "object" && val.length === 0));
			var node = (amountField.reqBkgdNode ? amountField.reqBkgdNode : amountField.domNode);
			domClass.toggle(node, "pinkBackground", !disabled && isNull);
		},

		updateModified: function(subClassValue) {
			var t = this;
			var modified = false;
			for (var id in t.amounts) {
				modified = true;
				break;
			}
			t.inherited(arguments,[modified]);
		},

		onAmountChanged: function(chargeId, value) {
			var t = this;
			var storeData = t.chargeGrid.store.get(chargeId);
			if ((storeData.amount == value) || isNaN(value)) {
				delete t.amounts[chargeId];
			} else {
				t.amounts[chargeId] = value;
			}
			
			var amountWidget = t.chargeGrid.cellWidget.getCellWidget(chargeId, "amount");
			if (amountWidget) {
				if (t._transferAllDeferreds[chargeId] && !t._transferAllDeferreds[chargeId].isResolved()) {
					t._transferAllDeferreds[chargeId].resolve();
				} else {	
					t.styleAmountField(amountWidget.amountField);
					t.updateTotalAmount();
				}
			}
			
			t.invalidate("modified");
		},

		onTransferChanged: function(chargeId, checked) {
			var t = this;
			var storeData = t.chargeGrid.store.get(chargeId);
			var amountWidget = t.chargeGrid.cellWidget.getCellWidget(chargeId, "amount");
			if (amountWidget) {
				var amountField = amountWidget.amountField;
				
				amountField.set("value", checked ? storeData.balance : 0);
				amountField.set("disabled", !checked);

				var deferred = t._transferAllDeferreds[chargeId];
				if (!deferred || deferred.isResolved()) {
					// If we're not updating because of "Transfer All Charges", select the amount
					_TextBoxMixin.selectInputText(amountField.textbox);
				}
				t.invalidate("modified");
			}

			// Un-check "Transfer All Charges" if user un-checked a Transfer box
			var transferAllCharges = t.getField("transferAllCharges");
			if (!checked && transferAllCharges.get("checked")) {
				transferAllCharges._onChangeActive = false;
				transferAllCharges.set("value", false);
				transferAllCharges._onChangeActive = true;
			}
		},

		updateTotalAmount: function() {
			var t = this;
			var totalAmount = 0;
			arrayUtils.forEach(t.chargeGrid.rows(), function(row) {
				amountWidget = row.grid.cellWidget.getCellWidget(row.id, "amount");
				if (amountWidget) {
					var chargeAmount = amountWidget.amountField.get("value");
					totalAmount += isNaN(chargeAmount) ? 0 : chargeAmount;
				}
			});
			t.getField("totalAmount").set("value", totalAmount);
		},

		_setValueAttr: function(value) {
			var t = this;
			if (value != null) {
				value.accountStatus				= value.account.status;
				value.dcn						= value.account.dcn;
				value.followUpDate				= value.account.followUpDate;
				value.followUpUser				= value.account.followUpUserId;

				arrayUtils.map(value.policies, function(policy) {
					policy.name = policy.insurance + " - " + policy.effectiveDate;
				});
			}
			t.inherited(arguments, [value]);
			t.amounts = {};
		},

		getSaveData: function() {
			var t = this;
			var value = t.get("value");
			value.id = t.props.value.id;
			value.amounts = t.amounts;
			return value;
		},

		resize: function() {
			var t = this;
			t.chargeGrid.resize();
		}
	});
});